package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysMessageService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysMessage;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description : 消息管理服务
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-25 19:03
 * @Modify :
 **/
@RestController
@RequestMapping("/sys-message")
@Api(description = "消息管理服务")
public class SysMessageController {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysMessageController.class);

    @Autowired
    private ISysMessageService sysMessageService;

    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增消息",notes = "新增消息")
    public ResultBean insertSysMessage(SysMessage sysMessage){
        try {
            // 登录用户
            SessionUser user = AuthUtil.getSessionUser();

            // 条件校验
            ValidatorBean.beginValid(sysMessage)
                    .notNull("messageTitle",sysMessage.getMessageTitle())
                    .notNull("messageContent",sysMessage.getMessageContent())
                    .checkNotZero("messageType",sysMessage.getMessageType())
                    .notNull("messageRecipientId",sysMessage.getMessageRecipientId());

            // 添加初始化
            ConvertBean.modelInitialize(sysMessage,user);
            // sysMessage.setMessageSendId(user.getUser().getId());
            // sysMessage.setRedSendName(user.getUser().getName());
            sysMessage.setMessageStatus(ImppEnumUtil.USER_MESSAGE_STATUS.UNREAD.getValue());
            // TODO 状态 status

            sysMessageService.insertSysMessage(sysMessage);
            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @DeleteMapping("/delete/{id}")
    @ApiOperation(value = "根据id删除消息",notes = "根据id删除消息")
    public ResultBean deleteSysMessageById(@PathVariable("id") String id){
        try {
            // 条件校验
            ValidatorBean.checkNotNull(id,"id不能为空");

            sysMessageService.deleteSysMessageById(id);
            return  ResultBean.success("删除成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping("/update")
    @ApiOperation(value = "修改信息",notes = "修改信息")
    public ResultBean updateSysMessage(SysMessage sysMessage){
        try {
            // 登录用户
            SessionUser user = AuthUtil.getSessionUser();

            // 条件校验
            ValidatorBean.beginValid(sysMessage)
                    .notNull("messageTitle",sysMessage.getMessageTitle())
                    .notNull("messageContent",sysMessage.getMessageContent())
                    .checkNotZero("messageType",sysMessage.getMessageType())
                    .notNull("messageRecipientId",sysMessage.getMessageRecipientId());

            // 添加初始化
            ConvertBean.modelUpdate(sysMessage,user);
            // sysMessage.setMessageSendId(user.getUser().getId());
            // sysMessage.setRedSendName(user.getUser().getName());
            // TODO 状态 status

            sysMessageService.updateSysMessage(sysMessage);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/list")
    @ApiOperation(value = "查询全部消息",notes = "查询全部消息")
    public ResultBean findSysConfigAll(){
        try {
            List<SysMessage> sysMessageList = sysMessageService.listSysMessage();
            return  ResultBean.success("查询成功")
                    .setResultList(sysMessageList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/get-by-id")
    @ApiOperation(value = "根据id查询系统参数",notes = "根据id查询系统参数")
    public ResultBean getSysConfigById(String id){
        try {
            SysMessage sysMessageList = sysMessageService.getSysMessageById(id);
            return  ResultBean.success("查询成功")
                    .setResultObject(sysMessageList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

}
