package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ITaskTimeExpressionService;
import cn.estsh.i3plus.core.api.iservice.busi.ITaskTimeService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description : 定时任务服务
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 16:58:43.982
 * @Modify      :
 **/
@RestController
@RequestMapping("/task-time")
@Api(description = "定时任务服务")
public class TaskTimeController {

    public static final Logger LOGGER = LoggerFactory.getLogger(TaskTimeController.class);

    @Autowired
    private ITaskTimeService taskTimeService;

    @Autowired
    private ITaskTimeExpressionService taskTimeExpressionService;

    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增定时任务", notes = "定时任务")
    public ResultBean insertTaskTime(TaskTime taskTime) {
        try {
            // 数据校验
            ValidatorBean.beginValid(taskTime)
                    .notNull("timeId", taskTime.getTimeId())
                    .notNull("name", taskTime.getName())
                    .notNull("taskDescription", taskTime.getTaskDescription());

            TaskTimeExpression taskTimeExpression = taskTimeExpressionService.getTaskTimeExpressionById(taskTime.getTimeId().toString());
            ValidatorBean.beginValid(taskTimeExpression);

            taskTime.setRedTimeName(taskTimeExpression.getName());
            taskTime.setRedTimeExpression(taskTimeExpression.getTimeExpression());

            ConvertBean.modelInitialize(taskTime, AuthUtil.getSessionUser());

            taskTime  = taskTimeService.insertTaskTime(taskTime);

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(taskTime);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "修改定时任务", notes = "定时任务")
    public ResultBean updateTaskTime(TaskTime taskTime) {
        try {
            // 数据校验
            ValidatorBean.beginValid(taskTime)
                    .notNull("id", taskTime.getId())
                    .notNull("timeId", taskTime.getTimeId())
                    .notNull("name", taskTime.getName())
                    .notNull("taskDescription", taskTime.getTaskDescription());

            TaskTimeExpression taskTimeExpression = taskTimeExpressionService.getTaskTimeExpressionById(taskTime.getTimeId().toString());

            // 数据真实性校验
            ValidatorBean.beginValid(taskTimeExpression);

            // 数据冗余
            taskTime.setRedTimeName(taskTimeExpression.getName());
            taskTime.setRedTimeExpression(taskTimeExpression.getTimeExpression());

            ConvertBean.modelUpdate(taskTime, AuthUtil.getSessionUser());

            taskTimeService.updateTaskTime(taskTime);

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(taskTime);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除定时任务", notes = "删除定时任务")
    public ResultBean deleteTaskTime(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "定时任务id 不能为空");

            taskTimeService.deleteTaskTimeById(id);
            return ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/list")
    @ApiOperation(value = "查询TaskTime", notes = "查询所有定时任务")
    public ResultBean findTaskTimeAll() {
        try {
            List<TaskTime> list = taskTimeService.listTaskTime();
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "查询定时任务", notes = "查询根据ID查询定时任务")
    public ResultBean getByTaskTimeId(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "定时任务id 不能为空");

            TaskTime taskTime = taskTimeService.getTaskTimeById(id);

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(taskTime);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/status")
    @ApiOperation(value = "修改定时任务状态", notes = "根据ID修改定时任务状态")
    public ResultBean updateStatus(String id,int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "功能id 不能为空");
            ValidatorBean.checkNotZero(status, "功能状态不能为空");

            taskTimeService.updateTaskTimeStatus(id,status,AuthUtil.getSessionUser());
            return new ResultBean(true);
        } catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }
}
