package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ICoreTreeService;
import cn.estsh.i3plus.pojo.platform.bean.Department;
import cn.estsh.i3plus.pojo.platform.bean.Position;
import cn.estsh.i3plus.pojo.platform.repository.DepartmentRepository;
import cn.estsh.i3plus.pojo.platform.repository.PositionRepository;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import javafx.scene.layout.VBox;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 通用树实现类
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-24 11:17
 * @Modify :
 **/
@Service
public class CoreTreeServiceImpl implements ICoreTreeService {

    public static final Logger LOGGER = LoggerFactory.getLogger(CoreTreeServiceImpl.class);

    @Autowired
    private DepartmentRepository departmentRDao;

    @Autowired
    private PositionRepository positionRDao;

    @Override
    public Department getDepartmentTreeByParentId(long depParentId) {
        Department depParent = departmentRDao.getById(depParentId);
        //封装子集
        findDepartmentChildrenTreePack(depParent, 0);
        return depParent;
    }

    /**
     * 封装部门子集
     *
     * @param parent
     */
    @Override
    public void findDepartmentChildrenTreePack(Department parent, int step) {
        ++step;
        List<Department> childDepList = departmentRDao.findByProperty("parentId", parent.getId());

        if (childDepList.size() > 0) {
            //说明有子集
            for (Department depChild : childDepList) {
                findDepartmentChildrenTreePack(depChild, step);
            }
        }

        LOGGER.info("【{}】包含子集：{},步长：{}", parent.getName(), childDepList.size(), step);
        parent.setChildList(childDepList);
    }


    @Override
    public List<Position> findPositionTreeByParentId(long parentId) {
        List<Position> result = positionRDao.findByProperty("parentId", parentId);

        // 循环设置子集
        if(result != null && result.size() > 0){

            result.stream().forEach(position -> findPositionChildrenTreePack(position,0));
        }

        return result;
    }

    @Override
    public Position getPositionTreeByParentId(long id) {
        // 查找数据
        Position position = positionRDao.getById(id);
        // 数据是否真实存在
        ValidatorBean.checkNotNull(position,"不存在的岗位信息");
        // 递归查询子集
        findPositionChildrenTreePack(position, 0);

        return position;
    }

    @Override
    public void findPositionChildrenTreePack(Position parent, int step) {
        ++step;
        List<Position> childList = positionRDao.findByProperty("parentId", parent.getId());

        if (childList.size() > 0) {
            //说明有子集
            for (Position child : childList) {
                findPositionChildrenTreePack(child, step);
            }
        }

        LOGGER.info("【{}】包含子集：{},步长：{}", parent.getName(), childList.size(), step);
        parent.setChildList(childList);
    }
}
