package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IDepartmentService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.Department;
import cn.estsh.i3plus.pojo.platform.bean.Organize;
import cn.estsh.i3plus.pojo.platform.repository.DepartmentRepository;
import cn.estsh.i3plus.pojo.platform.repository.OrganizeRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;

/**
 * @Description : 部门服务接口实现
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-24 20:47
 * @Modify :
 **/
@Service
public class DepartmentServiceImpl implements IDepartmentService {
    public static final Logger LOGGER = LoggerFactory.getLogger(DepartmentServiceImpl.class);

    @Autowired
    private DepartmentRepository departmentRDao;

    @Autowired
    private OrganizeRepository organizeRDao;

    @Override
    public void insertDepartment(Department department) {
        // 查询父级部门名称
        if (department.getParentId() != null  && department.getParentId() > 0) {
            LOGGER.info("部门信息 DEPARTMENT :{}", department.getParentId());
            Department parentPosition = departmentRDao.getById(department.getParentId());
            ValidatorBean.checkNotNull(parentPosition);

            department.setRedParentName(parentPosition.getDepartmentName());
        }

        // 查询所属组织名称
        LOGGER.info("组织信息 ORGANIZE :{}", department.getOrganizeId());
        Organize organize = organizeRDao.getById(department.getOrganizeId());
        ValidatorBean.checkNotNull(organize);

        department.setRedOrganizeName(organize.getName());

        // 新增部门
        LOGGER.info("部门信息 DEPARTMENT {}",department);
        departmentRDao.insert(department);
    }

    @Override
    public void deleteDepartmentById(String id) {
        // 查询是否存在子级
        LOGGER.info("部门信息 DEPARTMENT :{}", id);
        if (departmentRDao.findByPropertyCount("parentId",Long.parseLong(id)) > 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("存在子节点无法删除！")
                    .setErrorSolution("请重新操作")
                    .build();
        }

        // 删除部门
        LOGGER.info("部门信息 DEPARTMENT :{}", id);
        departmentRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public void updateDepartment(Department department) {
        // 查询父级部门名称
        if (department.getParentId() != null  && department.getParentId() > 0) {
            LOGGER.info("部门信息 DEPARTMENT :{}", department.getParentId());
            Department parentPosition = departmentRDao.getById(department.getParentId());
            ValidatorBean.checkNotNull(parentPosition);

            department.setRedParentName(parentPosition.getDepartmentName());
        }

        // 查询所属组织名称
        LOGGER.info("组织信息 ORGANIZE organizeId:{}", department.getOrganizeId());
        Organize organize = organizeRDao.getById(department.getOrganizeId());
        ValidatorBean.checkNotNull(organize);

        department.setRedOrganizeName(organize.getName());

        // 新增部门
        LOGGER.info("部门信息 DEPARTMENT department:{}",department);
        departmentRDao.update(department);
    }

    @Override
    public List<Department> listDepartment() {
        LOGGER.info("部门信息 DEPARTMENT list");
        return  departmentRDao.list();
    }

    @Override
    public Department getDepartmentById(String id) {
        LOGGER.info("部门信息 DEPARTMENT id:{}",id);
        return departmentRDao.getById(Long.parseLong(id));
    }

    @Override
    public ListPager<Department> findDepartmentByPager(Department department, Pager pager) {
        if (department == null) {
            pager = PagerHelper.getPager(pager, departmentRDao.listCount());
            // TODO 分页日志格式 排序
            return new ListPager(departmentRDao.listPager(pager),pager);
        } else {
            String hqlPack = CoreHqlPack.packHqlPosition(department);
            pager = PagerHelper.getPager(pager, departmentRDao.findByHqlWhereCount(hqlPack));
            // TODO 分页日志格式 排序
            return new ListPager(departmentRDao.findByHqlWherePage(hqlPack + department.getOrderByParam(),pager),pager);
        }
    }

}
